/* eslint-disable no-console */
import {
  Metadata, Loader, Router, Http, IDictionary,
} from '@zeedhi/core';
import {
  Text, Container, Form, AlertService, TextInput, Select, Grid, MenuGroup, Button, Checkbox, DialogService, MenuLink, LoadingService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class PerfilDeAcessoController {
  private ModalPAcessoController = Loader.getInstance('ModalPAcessoController');

  // Verifica qual o acesso e altera a exibição dos elementos
  public isVisibleElement: boolean = !(localStorage.getItem('idpermisssusuario') === 'V');

  public isDisabledElement: boolean = !(this.isVisibleElement);

  // Exclusão Realizada - nao necessaria
  public perfAcessDeleted() {
    AlertService.show({
      name: 'positionsDeleted',
      color: 'success',
      text: 'Todas as informações selecionadas foram corretamente apagadas!',
      top: true,
      left: false,
    });
  }

  public visibilidadeAcoesGrid({ component }: any) {
    Metadata.getInstance<Button>(`${component.name}_addButton`).disabled = this.isVisibleElement;
    Metadata.getInstance<Button>(`${component.name}_deleteButton`).disabled = this.isVisibleElement;
  }

  // Verifica qual o acesso e altera a exibição do menu
  public menuVisibility() {
    const configuracoesGroup = Metadata.getInstance<MenuGroup>('Configuracoes');
    const configuracoesItems = configuracoesGroup.items;
    const gerencialGroup = Metadata.getInstance<MenuGroup>('gerencial');
    const gerencialItems = gerencialGroup.items;
    const tarefasGroup = Metadata.getInstance<MenuGroup>('tarefas');
    const tarefasItems = tarefasGroup.items;
    const dashboadGenerico = Metadata.getInstance<MenuLink>('dashboard');
    const dashboadSupervisor = Metadata.getInstance<MenuLink>('dashboard_supervisor');
    const dashboadGerencial = Metadata.getInstance<MenuLink>('dashboard_gerencial');

    // const positionValue = localStorage.getItem('UserLogado');
    // const t1 = Metadata.getInstance<MenuGroup>('tarefas').items;
    // Http.get(`/getInfo/${positionValue}/PositionPermiss`).then((response: IDictionary) => {
    //   const database = response.data.dataset.info[0];

    const idpermissschecklist = localStorage.getItem('idpermissschecklist');
    const idpermisssusuario = localStorage.getItem('idpermisssusuario');
    const idpermisssgraficos = localStorage.getItem('idpermisssgraficos');
    const idpermissslocais = localStorage.getItem('idpermissslocais');
    const idpermisssrelatorios = localStorage.getItem('idpermisssrelatorios');
    const idpermissstarefas = localStorage.getItem('idpermissstarefas');
    const idpermisssvistoria = localStorage.getItem('idpermisssvistoria');
    const idpermissscalend = localStorage.getItem('idpermissscalend');
    const idpermisssDashboads = localStorage.getItem('idpermisssDashboads');

    if (idpermisssDashboads === 'O') {
      dashboadGenerico.isVisible = true;
    }

    if (idpermisssDashboads === 'S') {
      dashboadSupervisor.isVisible = true;
    }

    if (idpermisssDashboads === 'G') {
      dashboadGerencial.isVisible = true;
    }

    if (idpermisssvistoria === 'N' && idpermissscalend === 'N') {
      tarefasGroup.isVisible = false;
    } else {
      if (idpermisssvistoria === 'N') {
        tarefasItems[0].isVisible = false;
      }
      if (idpermissscalend === 'N') {
        tarefasItems[1].isVisible = false;
      }
    }

    if (idpermisssrelatorios === 'N' && idpermisssgraficos === 'N') {
      gerencialGroup.isVisible = false;
    } else {
      if (idpermisssrelatorios === 'N') {
        gerencialItems[1].isVisible = false;
      }
      if (idpermisssgraficos === 'N') {
        gerencialItems[2].isVisible = false;
      }
    }

    if (idpermissslocais === 'N' && idpermissschecklist === 'N' && idpermissstarefas === 'N' && idpermisssusuario === 'N') {
      configuracoesGroup.isVisible = false;
    } else {
      if (idpermissslocais === 'N') {
        configuracoesItems[1].isVisible = false;
      }
      if (idpermissschecklist === 'N') {
        configuracoesItems[2].isVisible = false;
      }
      if (idpermissstarefas === 'N') {
        configuracoesItems[3].isVisible = false;
      }
      if (idpermisssusuario === 'N') {
        configuracoesItems[4].isVisible = false;
      }
    }
    // });
  }

  // Exclusão não realizada
  public perfAcessNotDeleted() {
    DialogService.show({
      name: 'positionsDeleted',
      type: 'error',
      title: 'Informações não apagadas',
      text: 'Algumas informações não podem ser apagadas, pois possuem dependências!',
      buttons: [
        {
          name: 'ok',
          component: 'ZdButton',
          label: 'Ok',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  // Busca pelas permissões do usuário
  public async getPermissions() {
    const userId = localStorage.getItem('UserLogado');

    await Http.get(`/getInfo/${userId}/PositionPermiss`).then((response: IDictionary) => {
      const database = response.data.dataset.info[0];
      if (localStorage.getItem('Token') === database.token) {
        window.localStorage.setItem('idpermisssusuario', database.IDPERMISSSUSUARIO);
        window.localStorage.setItem('idpermissstarefas', database.IDPERMISSSTAREFAS);
        window.localStorage.setItem('idpermissslocais', database.IDPERMISSSLOCAIS);
        window.localStorage.setItem('idpermissschecklist', database.IDPERMISSSCHECKLIST);
        window.localStorage.setItem('idpermisssrelatorios', database.IDPERMISSSRELATORIOS);
        window.localStorage.setItem('idpermisssgraficos', database.IDPERMISSSGRAFICOS);
        window.localStorage.setItem('idpermisssvistoria', database.IDPERMISSSVISTORIA);
        window.localStorage.setItem('idpermissscalend', database.IDPERMISSSCALEND);
        window.localStorage.setItem('idpermisssDashboads', database.IDTIPODASHBOARD);
      } else {
        window.localStorage.clear();
        Router.replace('/Login');
      }
    });
    this.menuVisibility();
  }

  /* -------------------------------- Setores -------------------------------- */

  // data do Grid de locais vinculados
  public dataLinkedLocals(nrLocal: number, nmLocal: string) {
    return {
      value: nrLocal,
      local: nmLocal,
    };
  }

  // Abrir modal Novo Setor
  public openModalSetor() {
    this.ModalPAcessoController.openModalSetor();
  }

  // Abrir modal Editar setor
  public openToUpdateModalSetor({ row }: any) {
    this.ModalPAcessoController.userResposavel = row.NRFUNCRESPONSAVEL;
    Promise.all([this.openModalSetor()]).then(() => {
      const textObject = Metadata.getInstance<Text>('newSectorTitle');
      const nameInputObject = Metadata.getInstance<TextInput>('sectorName');
      const responsibleInputObject = Metadata.getInstance<Select>('userSelect');
      const saveButtonObject = Metadata.getInstance<Button>('sectorSaveButton');
      const cancelButtonObject = Metadata.getInstance<Button>('sectorCancelButton');

      nameInputObject.disabled = this.isDisabledElement;
      responsibleInputObject.disabled = this.isDisabledElement;
      saveButtonObject.disabled = this.isDisabledElement;
      cancelButtonObject.disabled = this.isDisabledElement;

      textObject.text = 'Dados do Setor';

      const containerObject = Metadata.getInstance<Container>('updateSectorContainer');
      containerObject.isVisible = true;

      const gridObject = Metadata.getInstance<TekGrid>('grid-setores');
      const sectorInfos = gridObject.datasource.currentRow;
      console.log(sectorInfos);
      // this.setSectorToUpdate(sectorInfos);
      Metadata.getInstance<Text>('nrsetorST').text = row.NRSETOR;
      Metadata.getInstance<TextInput>('sectorName').value = row.NMSETOR;
      this.getLinkedLocals(sectorInfos);
    });
  }

  // Valida sectorForm
  public sectorFormValidation() {
    const formObject = Metadata.getInstance<Form>('newSectorForm');
    const sectorTitle = Metadata.getInstance<Text>('newSectorTitle');

    if (formObject.validate()) {
      if (sectorTitle.text === 'Novo Setor') {
        this.createSector();
      } else {
        this.updateSector();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos',
        color: 'error',
      });
    }
  }

  // Cria um novo setor
  public createSector() {
    const sectorInfos = {
      NMSETOR: Metadata.getInstance<TextInput>('sectorName').value,
      NRFUNCRESPONSAVEL: Metadata.getInstance<Select>('userSelect').value,
    };
    const grid = Metadata.getInstance<TekGrid>('grid-setores');

    LoadingService.show();
    Http.post('/createSector', { sectorInfos }).then(() => {
      Utilities.showSuccessMessage();
      this.ModalPAcessoController.closeModalSetor();
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Edita setor
  public updateSector() {
    const params = {
      NRSETOR: Metadata.getInstance<Text>('nrsetorST').text,
      NMSETOR: Metadata.getInstance<TextInput>('sectorName').value,
      NRFUNCRESPONSAVEL: Metadata.getInstance<Select>('userSelect').value,
    };
    const grid = Metadata.getInstance<TekGrid>('grid-setores');

    LoadingService.show();
    Http.post('/updateSector', { params }).then(() => {
      Utilities.showSuccessMessage();
      grid.datasource.get();
      this.ModalPAcessoController.closeModalSetor();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Deleta setores
  public deleteSectors() {
    let validResponse: number;
    const grid = Metadata.getInstance<TekGrid>('grid-setores');
    const NRS = (grid.selectedRows.map((element: any) => element.NRSETOR));

    LoadingService.show();
    Http.post('/deleteSectors', {
      NRSETOR: NRS,
    }).then((response: IDictionary) => {
      validResponse = response.data;
      if (validResponse > 0) {
        this.perfAcessNotDeleted();
      } else {
        Utilities.showSuccessMessage();
      }
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Caixa de confirmação de delete
  public confirmSectorsDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Setor',
      text: 'Todas as informações deste(s) Setor(es) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.deleteSectors();
              DialogService.hide();
            },
          },
          outline: true,
        },
      ],
    });
  }

  // Busca Por locais Vinculados ao setor
  public getLinkedLocals(info: IDictionary) {
    const gridObject = Metadata.getInstance<Grid>('sectorLocalGrid');
    const params = info.NRSETOR;

    Http.get(`/getLinkedLocals/${params}/locals`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        gridObject.datasource.data.push(this.dataLinkedLocals(database.NRLOCALCLIENTE, database.NMLOCALCLIENTE));
      });
    });
  }

  /* -------------------------------- Funções -------------------------------- */

  // input data pesquisar
  public dataFilter: IDictionary = {
    positionSearch: '',
    sectorSearch: '',
    userSearch: '',
  };

  // Condições das checkbox para o BD
  public readOrWriteData: IDictionary = {
    users: '',
    inspections: '',
    tasks: '',
    locals: '',
    checklists: '',
    reports: '',
    graphs: '',
    calendar: '',
  };

  // Abrir modal Nova Função
  public openModalFuncao() {
    this.ModalPAcessoController.openModalFuncao();
  }

  // Abrir modal Editar Função (Não está sendo mais usado)
  public openToUpdateModalFuncao({ row }: any) {
    Promise.all([this.openModalFuncao()]).then(() => {
      const textObject = Metadata.getInstance<Text>('newPositionTitle');
      const saveButtonObject = Metadata.getInstance<Button>('positionSaveButton');
      const cancelButtonObject = Metadata.getInstance<Button>('positionCancelButton');

      Metadata.getInstance<TextInput>('positionName').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('positionDashboard').disabled = this.isDisabledElement;
      Metadata.getInstance<Select>('positionQRCode').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('usersVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('usersECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('inspectionsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('inspectionsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('tasksVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('tasksECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('localsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('localsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('checklistsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('checklistsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('reportsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('reportsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('graphsVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('graphsECheck').disabled = this.isDisabledElement;

      Metadata.getInstance<Checkbox>('calendarVCheck').disabled = this.isDisabledElement;
      Metadata.getInstance<Checkbox>('calendarECheck').disabled = this.isDisabledElement;

      saveButtonObject.disabled = this.isDisabledElement;
      cancelButtonObject.disabled = this.isDisabledElement;

      textObject.text = 'Dados da Função';
      this.setPositionToUpdate(row);
    });
  }

  // Preenche array de acordo com o valor do array acima
  public databasePermissions() {
    const usersVObject = Metadata.getInstance<Checkbox>('usersVCheck').value;
    const usersEObject = Metadata.getInstance<Checkbox>('usersECheck').value;

    const inspectionsVObject = Metadata.getInstance<Checkbox>('inspectionsVCheck').value;
    const inspectionsEObject = Metadata.getInstance<Checkbox>('inspectionsECheck').value;

    const tasksVObject = Metadata.getInstance<Checkbox>('tasksVCheck').value;
    const tasksEObject = Metadata.getInstance<Checkbox>('tasksECheck').value;

    const localsVObject = Metadata.getInstance<Checkbox>('localsVCheck').value;
    const localsEObject = Metadata.getInstance<Checkbox>('localsECheck').value;

    const checklistsVObject = Metadata.getInstance<Checkbox>('checklistsVCheck').value;
    const checklistsEObject = Metadata.getInstance<Checkbox>('checklistsECheck').value;

    const reportsVObject = Metadata.getInstance<Checkbox>('reportsVCheck').value;
    const reportsEObject = Metadata.getInstance<Checkbox>('reportsECheck').value;

    const graphsVObject = Metadata.getInstance<Checkbox>('graphsVCheck').value;
    const graphsEObject = Metadata.getInstance<Checkbox>('graphsECheck').value;

    const calendarVObject = Metadata.getInstance<Checkbox>('calendarVCheck').value;
    const calendarEObject = Metadata.getInstance<Checkbox>('calendarECheck').value;

    if ((usersVObject && usersEObject) || usersEObject) {
      this.readOrWriteData.users = 'E';
    } else if (usersVObject) {
      this.readOrWriteData.users = 'V';
    } else this.readOrWriteData.users = 'N';

    if ((inspectionsVObject && inspectionsEObject) || inspectionsEObject) {
      this.readOrWriteData.inspections = 'E';
    } else if (inspectionsVObject) {
      this.readOrWriteData.inspections = 'V';
    } else this.readOrWriteData.inspections = 'N';

    if ((tasksVObject && tasksEObject) || tasksEObject) {
      this.readOrWriteData.tasks = 'E';
    } else if (tasksVObject) {
      this.readOrWriteData.tasks = 'V';
    } else this.readOrWriteData.tasks = 'N';

    if ((localsVObject && localsEObject) || localsEObject) {
      this.readOrWriteData.locals = 'E';
    } else if (localsVObject) {
      this.readOrWriteData.locals = 'V';
    } else this.readOrWriteData.locals = 'N';

    if ((checklistsVObject && checklistsEObject) || checklistsEObject) {
      this.readOrWriteData.checklists = 'E';
    } else if (checklistsVObject) {
      this.readOrWriteData.checklists = 'V';
    } else this.readOrWriteData.checklists = 'N';

    if ((reportsVObject && reportsEObject) || reportsEObject) {
      this.readOrWriteData.reports = 'E';
    } else if (reportsVObject) {
      this.readOrWriteData.reports = 'V';
    } else this.readOrWriteData.reports = 'N';

    if ((graphsVObject && graphsEObject) || graphsEObject) {
      this.readOrWriteData.graphs = 'E';
    } else if (graphsVObject) {
      this.readOrWriteData.graphs = 'V';
    } else this.readOrWriteData.graphs = 'N';

    if ((calendarVObject && calendarEObject) || calendarEObject) {
      this.readOrWriteData.calendar = 'E';
    } else if (calendarVObject) {
      this.readOrWriteData.calendar = 'V';
    } else this.readOrWriteData.calendar = 'N';
  }

  // Prenche o array para edição
  public setPermissions(idpermisssusuario: string, idpermisssvistoria: string, idpermissstarefas: string, idpermissslocais: string, idpermissschecklist: string, idpermisssrelatorios: string, idpermisssgraficos: string, idpermissscalend: string) {
    const usersVObject = Metadata.getInstance<Checkbox>('usersVCheck');
    const usersEObject = Metadata.getInstance<Checkbox>('usersECheck');

    const inspectionsVObject = Metadata.getInstance<Checkbox>('inspectionsVCheck');
    const inspectionsEObject = Metadata.getInstance<Checkbox>('inspectionsECheck');

    const tasksVObject = Metadata.getInstance<Checkbox>('tasksVCheck');
    const tasksEObject = Metadata.getInstance<Checkbox>('tasksECheck');

    const localsVObject = Metadata.getInstance<Checkbox>('localsVCheck');
    const localsEObject = Metadata.getInstance<Checkbox>('localsECheck');

    const checklistsVObject = Metadata.getInstance<Checkbox>('checklistsVCheck');
    const checklistsEObject = Metadata.getInstance<Checkbox>('checklistsECheck');

    const reportsVObject = Metadata.getInstance<Checkbox>('reportsVCheck');
    const reportsEObject = Metadata.getInstance<Checkbox>('reportsECheck');

    const graphsVObject = Metadata.getInstance<Checkbox>('graphsVCheck');
    const graphsEObject = Metadata.getInstance<Checkbox>('graphsECheck');

    const calendarVObject = Metadata.getInstance<Checkbox>('calendarVCheck');
    const calendarEObject = Metadata.getInstance<Checkbox>('calendarECheck');

    if (idpermisssusuario === 'A' || idpermisssusuario === 'E') {
      usersVObject.value = true;
      usersEObject.value = true;
    } else if (idpermisssusuario === 'V') {
      usersVObject.value = true;
    } else {
      usersVObject.value = false;
      usersEObject.value = false;
    }

    if (idpermisssvistoria === 'A' || idpermisssvistoria === 'E') {
      inspectionsVObject.value = true;
      inspectionsEObject.value = true;
    } else if (idpermisssvistoria === 'V') {
      inspectionsVObject.value = true;
    } else {
      inspectionsVObject.value = false;
      inspectionsEObject.value = false;
    }

    if (idpermissstarefas === 'A' || idpermissstarefas === 'E') {
      tasksVObject.value = true;
      tasksEObject.value = true;
    } else if (idpermissstarefas === 'V') {
      tasksVObject.value = true;
    } else {
      tasksVObject.value = false;
      tasksEObject.value = false;
    }

    if (idpermissslocais === 'A' || idpermissslocais === 'E') {
      localsVObject.value = true;
      localsEObject.value = true;
    } else if (idpermissslocais === 'V') {
      localsVObject.value = true;
    } else {
      localsVObject.value = false;
      localsEObject.value = false;
    }

    if (idpermissschecklist === 'A' || idpermissschecklist === 'E') {
      checklistsVObject.value = true;
      checklistsEObject.value = true;
    } else if (idpermissschecklist === 'V') {
      checklistsVObject.value = true;
    } else {
      checklistsVObject.value = false;
      checklistsEObject.value = false;
    }

    if (idpermisssrelatorios === 'A' || idpermisssrelatorios === 'E') {
      reportsVObject.value = true;
      reportsEObject.value = true;
    } else if (idpermisssrelatorios === 'V') {
      reportsVObject.value = true;
    } else {
      reportsVObject.value = false;
      reportsEObject.value = false;
    }

    if (idpermisssgraficos === 'A' || idpermisssgraficos === 'E') {
      graphsVObject.value = true;
      graphsEObject.value = true;
    } else if (idpermisssgraficos === 'V') {
      graphsVObject.value = true;
    } else {
      graphsVObject.value = false;
      graphsEObject.value = false;
    }

    if (idpermissscalend === 'A' || idpermissscalend === 'E') {
      calendarVObject.value = true;
      calendarEObject.value = true;
    } else if (idpermissscalend === 'V') {
      calendarVObject.value = true;
    } else {
      calendarVObject.value = false;
      calendarEObject.value = false;
    }
  }

  // Cria uma nova Função
  public createPosition() {
    const grid = Metadata.getInstance<TekGrid>('grid-funcoes');

    this.databasePermissions();
    const params = {
      NMPERFILACESSO: Metadata.getInstance<TextInput>('positionName').value,
      IDPERMISSSUSUARIO: this.readOrWriteData.users,
      IDPERMISSSFUNCOES: this.readOrWriteData.positions,
      IDPERMISSSTAREFAS: this.readOrWriteData.tasks,
      IDPERMISSSLOCAIS: this.readOrWriteData.locals,
      IDPERMISSSCHECKLIST: this.readOrWriteData.checklists,
      IDPERMISSSRELATORIOS: this.readOrWriteData.reports,
      IDPERMISSSGRAFICOS: this.readOrWriteData.graphs,
      IDPERMISSSVISTORIA: this.readOrWriteData.inspections,
      IDPERMISSSCALEND: this.readOrWriteData.calendar,
      IDDIRECIONAQRCODE: Metadata.getInstance<Select>('positionQRCode').value,
      IDTIPODASHBOARD: Metadata.getInstance<Select>('positionDashboard').value,
    };
    LoadingService.show();
    Http.post('/createPosition', { params }).then(() => {
      Utilities.showSuccessMessage();
      grid.datasource.get();
      this.ModalPAcessoController.closeModalFuncao();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Seta os dados no modal para editar
  public setPositionToUpdate(database: IDictionary) {
    Metadata.getInstance<TextInput>('positionName').value = database.NMPERFILACESSO;
    Metadata.getInstance<Text>('nrperfilacesso').text = database.NRPERFILACESSO;
    Metadata.getInstance<Select>('positionQRCode').setValue(database.IDDIRECIONAQRCODE);
    Metadata.getInstance<Select>('positionDashboard').setValue(database.IDTIPODASHBOARD);
    this.setPermissions(database.IDPERMISSSUSUARIO, database.IDPERMISSSVISTORIA, database.IDPERMISSSTAREFAS, database.IDPERMISSSLOCAIS, database.IDPERMISSSCHECKLIST, database.IDPERMISSSRELATORIOS, database.IDPERMISSSGRAFICOS, database.IDPERMISSSCALEND);
  }

  // Edita a função
  public updatePosition() {
    const grid = Metadata.getInstance<TekGrid>('grid-funcoes');

    this.databasePermissions();
    LoadingService.show();
    const params = {
      NRPERFILACESSO: Metadata.getInstance<Text>('nrperfilacesso').text,
      NMPERFILACESSO: Metadata.getInstance<TextInput>('positionName').value,
      IDPERMISSSUSUARIO: this.readOrWriteData.users,
      IDPERMISSSFUNCOES: this.readOrWriteData.positions,
      IDPERMISSSTAREFAS: this.readOrWriteData.tasks,
      IDPERMISSSLOCAIS: this.readOrWriteData.locals,
      IDPERMISSSCHECKLIST: this.readOrWriteData.checklists,
      IDPERMISSSRELATORIOS: this.readOrWriteData.reports,
      IDPERMISSSGRAFICOS: this.readOrWriteData.graphs,
      IDPERMISSSVISTORIA: this.readOrWriteData.inspections,
      IDPERMISSSCALEND: this.readOrWriteData.calendar,
      IDDIRECIONAQRCODE: Metadata.getInstance<Select>('positionQRCode').value,
      IDTIPODASHBOARD: Metadata.getInstance<Select>('positionDashboard').value,
    };
    Http.post('/updatePosition', { params }).then(() => {
      Utilities.showSuccessMessage();
      this.ModalPAcessoController.closeModalFuncao();
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Deleta a função
  public deletePositions() {
    let validResponse: number;
    const grid = Metadata.getInstance<TekGrid>('grid-funcoes');
    const NRS = (grid.selectedRows.map((element: any) => element.NRPERFILACESSO));

    LoadingService.show();
    Http.post('/deletePositions', {
      NRPERFILACESSO: NRS,
    }).then((response: IDictionary) => {
      validResponse = response.data;
      if (validResponse > 0) {
        this.perfAcessNotDeleted();
      } else {
        Utilities.showSuccessMessage();
      }
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Confirmar se realiza a exclusão
  public confirmPositionsDelete({ event }: any) {
    event.preventDefault();

    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Função',
      text: 'Todas as informações desta(s) Função(ões) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          outline: true,
          events: {
            click: () => {
              this.deletePositions();
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  // Valida formulário Nova função
  public validatePositionForm() {
    const formObject = Metadata.getInstance<Form>('newPositionForm');
    const titleText = Metadata.getInstance<Text>('newPositionTitle');

    if (formObject.validate()) {
      if (titleText.text === 'Nova Função') {
        this.createPosition();
      } else {
        this.updatePosition();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos.',
        color: 'error',
      });
    }
  }

  /* -------------------------------- Usuários -------------------------------- */

  // Abrir modal Novo Usuário
  public openModalUsuario() {
    this.ModalPAcessoController.openModalUsuario();
  }

  // Abrir modal Editar Usuário
  public openToUpdateModalUsuario({ row }:any) {
    Promise.all([this.openModalUsuario()]).then(() => {
      this.ModalPAcessoController.textObject = 'Dados do Usuário';
      this.ModalPAcessoController.isDisabledElement = this.isDisabledElement;

      this.setUserToUpdate(row);
    });
  }

  // Valida userForm
  public userFormValidation() {
    const formObject = Metadata.getInstance<Form>('newUserForm');
    const textObject = Metadata.getInstance<Text>('titleTextNewEmployee');

    if (formObject.validate()) {
      if (textObject.text === 'Novo Usuário') {
        this.createUser();
      } else {
        this.updateUser();
      }
    } else {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Preencha todos os campos',
        color: 'error',
      });
    }
  }

  // Criar novo Usuário
  public createUser() {
    const userInfos = {
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('userName').value,
      NRPERFILACESSO: Metadata.getInstance<Select>('positionSelect').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('userCPF').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('userRG').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('userEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('userTel').value,
      NRSETOR: Metadata.getInstance<Select>('sectorSelect').value,
      CDOPERADOR: Metadata.getInstance<Select>('SelectOperador').value,
    };

    const grid = Metadata.getInstance<TekGrid>('grid-usuarios');

    LoadingService.show();
    Http.post('/createUser', { userInfos }).then(() => {
      Utilities.showSuccessMessage();
      this.ModalPAcessoController.closeModalUsuario();
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public setUserToUpdate(database: any) {
    this.ModalPAcessoController.nrfuncionario = database.NRFUNCIONARIO;
    this.ModalPAcessoController.userName = database.NMFUNCIONARIO;
    this.ModalPAcessoController.positionSelect = { NRPERFILACESSO: database.NRPERFILACESSO };
    this.ModalPAcessoController.relacfuncperfacess = database.RELACFUNCPERFACESS;
    this.ModalPAcessoController.userCPF = database.NRCPFFUNCIONARIO;
    this.ModalPAcessoController.userRG = database.NRRGFUNCIONARIO;
    this.ModalPAcessoController.userEmail = database.DSEMAILFUNCIONARIO;
    this.ModalPAcessoController.userTel = (database.NRTELFUNCIONARIO === null) ? ' ' : database.NRTELFUNCIONARIO.toString();
    this.ModalPAcessoController.sectorSelect = database.NRSETOR;
    this.ModalPAcessoController.nrrelacfuncsetor = database.NRRELACFUNCSETOR;
    this.ModalPAcessoController.SelectOperador = { CDOPERADOR: database.CDOPERADOR };
  }

  // Editar usuário
  public updateUser() {
    const params = {
      NRFUNCIONARIO: Metadata.getInstance<Text>('nrfuncionario').text,
      NMFUNCIONARIO: Metadata.getInstance<TextInput>('userName').value,
      NRPERFILACESSO: Metadata.getInstance<Select>('positionSelect').value,
      NRCPFFUNCIONARIO: Metadata.getInstance<TextInput>('userCPF').value,
      NRRGFUNCIONARIO: Metadata.getInstance<TextInput>('userRG').value,
      DSEMAILFUNCIONARIO: Metadata.getInstance<TextInput>('userEmail').value,
      NRTELFUNCIONARIO: Metadata.getInstance<TextInput>('userTel').value,
      NRSETOR: Metadata.getInstance<Select>('sectorSelect').value,
      RELACFUNCPERFACESS: Metadata.getInstance<Text>('relacfuncperfacess').text,
      NRRELACFUNCSETOR: Metadata.getInstance<Text>('nrrelacfuncsetor').text,
      CDOPERADOR: Metadata.getInstance<Select>('SelectOperador').value,
    };

    const grid = Metadata.getInstance<TekGrid>('grid-usuarios');

    LoadingService.show();
    Http.post('/updateUser', { params }).then(() => {
      Utilities.showSuccessMessage();
      this.ModalPAcessoController.closeModalUsuario();
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Deletar Usuário
  public deleteUsers() {
    let validResponse: any;
    const grid = Metadata.getInstance<TekGrid>('grid-usuarios');
    const NRS = (grid.selectedRows.map((element: any) => element.NRFUNCIONARIO));

    Http.post('/deleteUsers', {
      NRFUNCIONARIO: NRS,
    }).then((response: IDictionary) => {
      validResponse = response.data;
      if (validResponse > 0) {
        this.perfAcessNotDeleted();
      } else {
        Utilities.showSuccessMessage();
      }
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Confirmar se será feita a exclusão
  public confirmUserDelete() {
    DialogService.show({
      name: 'button-delete',
      type: 'warning',
      title: 'Excluir Usuário',
      text: 'Todas as informações deste(s) Usuário(s) serão perdidas. Deseja continuar?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: {
            click: () => {
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'yes-users',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.deleteUsers();
              DialogService.hide();
            },
          },
        },
      ],
    });
  }

  public gerarDadosModal() {
    this.ModalPAcessoController.gerarDadosModal();
  }
}
